'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createDefaultActionTypes = createDefaultActionTypes;
exports.pouchdbAction = pouchdbAction;
exports.allDocs = allDocs;
exports.query = query;
exports.get = get;
exports.put = put;
exports.post = post;
exports.remove = remove;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _constants = require('../constants');

var _utils = require('../utils');

var _middleware = require('../middleware');

/**
 * Utility function returns array of 3 action types with defaults.
 *
 * @param {string} action
 * @param {Object} actionTypes object with `request`, `success`, `failure` keys,
 * or string that represent `success`
 */

function createDefaultActionTypes(action) {
  var actionTypes = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var types = _extends({
    request: (0, _utils.createActionType)('', action, _constants.TYPES.request),
    success: (0, _utils.createActionType)('', action, _constants.TYPES.success),
    failure: (0, _utils.createActionType)('', action, _constants.TYPES.failure)
  }, actionTypes);
  return [types.request, types.success, types.failure];
}

/*
 * Creates pouchdb action.
 *
 * `method` - pouchdb service method
 * `types` - action types
 * `params` - params to delegate to pouchdb service method
 * `opts` - additional options to add to action
*/

function pouchdbAction(method, types) {
  var params = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  return _extends(_defineProperty({}, _middleware.POUCHDB, {
    types: types,
    method: method,
    params: params
  }), opts);
}

/*
 * Creates action for `allDocs`
*/

function allDocs() {
  var types = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var params = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  return pouchdbAction(_constants.ACTIONS.allDocs, createDefaultActionTypes(_constants.ACTIONS.allDocs, types), params, opts);
}

/*
 * Creates action for `allDocs`
*/

function query() {
  var types = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var params = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  return pouchdbAction(_constants.ACTIONS.query, createDefaultActionTypes(_constants.ACTIONS.query, types), params, opts);
}

/*
 * Creates action for `get`
*/

function get(docId) {
  var types = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var params = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  return pouchdbAction(_constants.ACTIONS.get, createDefaultActionTypes(_constants.ACTIONS.get, types), _extends({ docId: docId }, params), opts);
}

/*
 * Creates action for `put`
 */

function put(doc) {
  var types = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var params = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  return pouchdbAction(_constants.ACTIONS.put, createDefaultActionTypes(_constants.ACTIONS.put, types), _extends({ doc: doc }, params), opts);
}

/*
 * Creates action for `post`
 */

function post(doc) {
  var types = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var params = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  return pouchdbAction(_constants.ACTIONS.post, createDefaultActionTypes(_constants.ACTIONS.post, types), _extends({ doc: doc }, params), opts);
}

/*
 * Creates action for `remove`
 */

function remove(doc) {
  var types = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var params = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var opts = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  return pouchdbAction(_constants.ACTIONS.remove, createDefaultActionTypes(_constants.ACTIONS.remove, types), _extends({ doc: doc }, params), opts);
}